-- SQL Consolidated database with SQL Anywhere remote using TCPIP
-- SQL Anywhere Studio MobiLink Synchronization 10.0.1
-- Written by Joshua Savill
-- Product Manager - Sybase iAnywhere
-- January 23, 2008

/*------------------------------------------------------------------------------
*                           Set some Database Options.
*-----------------------------------------------------------------------------*/

set option public.Global_database_id = 1;

/*------------------------------------------------------------------------------
*               Creating schema related to the table 'DBA.Admin'.
*-----------------------------------------------------------------------------*/

/* Create base Admin table */
create table Admin (
  admin_id  bigint default global autoincrement primary key,
  data  varchar(30),
  last_modified timestamp not null default current timestamp
);

/* Create the shadow delete table 'DBA.Admin_del'. */
CREATE TABLE "DBA"."Admin_del" (
	"admin_id" bigint not null,
	"last_modified" TIMESTAMP DEFAULT CURRENT TIMESTAMP,
	PRIMARY KEY ("admin_id")
);

/* Create the shadow delete trigger 'Admin_ins'. */
CREATE TRIGGER "Admin_ins" AFTER INSERT ORDER 3
ON "DBA"."Admin"
REFERENCING
	NEW AS inserted
FOR EACH STATEMENT
BEGIN
	/*
	* Delete the row from the shadow delete table.  (This trigger is only needed if deleted
	* primary keys can be re-inserted.)
	*/
	DELETE FROM "DBA"."Admin_del"
	WHERE EXISTS (
		SELECT 1
		FROM inserted
		WHERE "DBA"."Admin_del"."admin_id" = inserted."admin_id"
		);
END;

/* Create the shadow delete trigger 'Admin_dlt'. */
CREATE TRIGGER "Admin_dlt" AFTER DELETE ORDER 2
ON "DBA"."Admin"
REFERENCING
	OLD AS deleted
FOR EACH STATEMENT
BEGIN
	/* Insert the row into the shadow delete table. */
	INSERT INTO "DBA"."Admin_del" ( "admin_id", "last_modified" )
	SELECT deleted."admin_id",
		CURRENT TIMESTAMP
	FROM deleted;
END;

/* Create the timestamp trigger 'Admin_upd'. */
CREATE TRIGGER "Admin_upd" AFTER UPDATE
OF "admin_id", "data"
ORDER 1 ON "DBA"."Admin"
REFERENCING NEW AS inserted OLD AS deleted
FOR EACH STATEMENT
BEGIN

	/* Update the column last_modified in modified row. */
	UPDATE "DBA"."Admin"
	SET "last_modified" = CURRENT TIMESTAMP
	FROM inserted
	WHERE  "DBA"."Admin"."admin_id" = inserted."admin_id";
END;

/* Create an index for the 'download_cursor' script. */
CREATE INDEX "Admin_ml" ON "DBA"."Admin"	( "last_modified" );

COMMIT;


/*------------------------------------------------------------------------------
*              Creating schema related to the table 'DBA.Parent'.
*-----------------------------------------------------------------------------*/

/* Create base Parent table */
create table Parent (
  parent_id  bigint default global autoincrement primary key,
  user_id  varchar(128),
  data  varchar(30),
  last_modified timestamp not null default current timestamp
);

/* Create the shadow delete table 'DBA.Parent_del'. */
CREATE TABLE "DBA"."Parent_del" (
	"parent_id" bigint not null,
	"last_modified" TIMESTAMP DEFAULT CURRENT TIMESTAMP,
	PRIMARY KEY ("parent_id")
);

/* Create the shadow delete trigger 'Parent_ins'. */
CREATE TRIGGER "Parent_ins" AFTER INSERT ORDER 3
ON "DBA"."Parent"
REFERENCING
	NEW AS inserted
FOR EACH STATEMENT
BEGIN
	/*
	* Delete the row from the shadow delete table.  (This trigger is only needed if deleted
	* primary keys can be re-inserted.)
	*/
	DELETE FROM "DBA"."Parent_del"
	WHERE EXISTS (
		SELECT 1
		FROM inserted
		WHERE "DBA"."Parent_del"."parent_id" = inserted."parent_id"
		);
END;

/* Create the shadow delete trigger 'Parent_dlt'. */
CREATE TRIGGER "Parent_dlt" AFTER DELETE ORDER 2
ON "DBA"."Parent"
REFERENCING
	OLD AS deleted
FOR EACH STATEMENT
BEGIN
	/* Insert the row into the shadow delete table. */
	INSERT INTO "DBA"."Parent_del" ( "parent_id", "last_modified" )
	SELECT deleted."parent_id",
		CURRENT TIMESTAMP
	FROM deleted;
END;

/* Create the timestamp trigger 'Parent_upd'. */
CREATE TRIGGER "Parent_upd" AFTER UPDATE
OF "parent_id", "user_id", "data"
ORDER 1 ON "DBA"."Parent"
REFERENCING NEW AS inserted OLD AS deleted
FOR EACH STATEMENT
BEGIN

	/* Update the column last_modified in modified row. */
	UPDATE "DBA"."Parent"
	SET "last_modified" = CURRENT TIMESTAMP
	FROM inserted
	WHERE  "DBA"."Parent"."parent_id" = inserted."parent_id";
END;

/* Create an index for the 'download_cursor' script. */
CREATE INDEX "Parent_ml" ON "DBA"."Parent" ( "last_modified" );

COMMIT;

/*------------------------------------------------------------------------------
*               Creating schema related to the table 'DBA.Child'.
*-----------------------------------------------------------------------------*/

/* Create base Child table */
create table Child (
  child_id bigint default global autoincrement primary key,
  parent_id bigint references Parent,
  data varchar( 30 ),
  last_modified timestamp not null default current timestamp
);

/* Create the shadow delete table 'DBA.Child_del'. */
CREATE TABLE "DBA"."Child_del" (
	"child_id" bigint not null,
	"last_modified" TIMESTAMP DEFAULT CURRENT TIMESTAMP,
	PRIMARY KEY ("child_id")
);

/* Create the shadow delete trigger 'Child_ins'. */
CREATE TRIGGER "Child_ins" AFTER INSERT ORDER 3
ON "DBA"."Child"
REFERENCING
	NEW AS inserted
FOR EACH STATEMENT
BEGIN
	/*
	* Delete the row from the shadow delete table.  (This trigger is only needed if deleted
	* primary keys can be re-inserted.)
	*/
	DELETE FROM "DBA"."Child_del"
	WHERE EXISTS (
		SELECT 1
		FROM inserted
		WHERE "DBA"."Child_del"."child_id" = inserted."child_id"
		);
END;

/* Create the shadow delete trigger 'Child_dlt'. */
CREATE TRIGGER "Child_dlt" AFTER DELETE ORDER 2
ON "DBA"."Child"
REFERENCING
	OLD AS deleted
FOR EACH STATEMENT
BEGIN
	/* Insert the row into the shadow delete table. */
	INSERT INTO "DBA"."Child_del" ( "child_id", "last_modified" )
	SELECT deleted."child_id",
		CURRENT TIMESTAMP
	FROM deleted;
END;

/* Create the timestamp trigger 'Child_upd'. */
CREATE TRIGGER "Child_upd" AFTER UPDATE
OF "child_id", "parent_id", "data"
ORDER 1 ON "DBA"."Child"
REFERENCING NEW AS inserted OLD AS deleted
FOR EACH STATEMENT
BEGIN

	/* Update the column last_modified in modified row. */
	UPDATE "DBA"."Child"
	SET "last_modified" = CURRENT TIMESTAMP
	FROM inserted
	WHERE  "DBA"."Child"."child_id" = inserted."child_id";
END;

/* Create an index for the 'download_cursor' script. */
CREATE INDEX "Child_ml" ON "DBA"."Child" ( "last_modified" );

COMMIT;

/*------------------------------------------------------------------------------
*             Creating schema related to the table 'DBA.Inventory'.
*-----------------------------------------------------------------------------*/

/* Create base Inventory table */
create table Inventory (
  inventory_id bigint default global autoincrement primary key,
  data  integer,
  last_modified timestamp not null default current timestamp
);

/* Create the shadow delete table 'DBA.Inventory_del'. */
CREATE TABLE "DBA"."Inventory_del" (
	"inventory_id" bigint not null,
	"last_modified" TIMESTAMP DEFAULT CURRENT TIMESTAMP,
	PRIMARY KEY ("inventory_id")
);

/* Create the shadow delete trigger 'Inventory_ins'. */
CREATE TRIGGER "Inventory_ins" AFTER INSERT ORDER 3
ON "DBA"."Inventory"
REFERENCING
	NEW AS inserted
FOR EACH STATEMENT
BEGIN
	/*
	* Delete the row from the shadow delete table.  (This trigger is only needed if deleted
	* primary keys can be re-inserted.)
	*/
	DELETE FROM "DBA"."Inventory_del"
	WHERE EXISTS (
		SELECT 1
		FROM inserted
		WHERE "DBA"."Inventory_del"."inventory_id" = inserted."inventory_id"
		);
END;

/* Create the shadow delete trigger 'Inventory_dlt'. */
CREATE TRIGGER "Inventory_dlt" AFTER DELETE ORDER 2
ON "DBA"."Inventory"
REFERENCING
	OLD AS deleted
FOR EACH STATEMENT
BEGIN
	/* Insert the row into the shadow delete table. */
	INSERT INTO "DBA"."Inventory_del" ( "inventory_id", "last_modified" )
	SELECT deleted."inventory_id",
		CURRENT TIMESTAMP
	FROM deleted;
END;

/* Create the timestamp trigger 'Inventory_upd'. */
CREATE TRIGGER "Inventory_upd" AFTER UPDATE
OF "inventory_id", "data"
ORDER 1 ON "DBA"."Inventory"
REFERENCING NEW AS inserted OLD AS deleted
FOR EACH STATEMENT
BEGIN

	/* Update the column last_modified in modified row. */
	UPDATE "DBA"."Inventory"
	SET "last_modified" = CURRENT TIMESTAMP
	FROM inserted
	WHERE  "DBA"."Inventory"."inventory_id" = inserted."inventory_id";
END;

/* Create an index for the 'download_cursor' script. */
CREATE INDEX "Inventory_ml" ON "DBA"."Inventory" ( "last_modified" );

COMMIT;

/* Add the connection scripts */

-- CALL ml_add_connection_script (
--	'https_v1',
--	'begin_connection',
--	''
--);

-- CALL ml_add_connection_script (
--	'https_v1',
--	'authenticate_user',
--	''
--);

-- CALL ml_add_connection_script (
--	'https_v1',
--	'end_publication',
--	''
--);

CALL ml_add_table_script (
	'https_v1',
	'Admin',
	'download_cursor',
'
SELECT "DBA"."Admin"."admin_id",
	"DBA"."Admin"."data"
FROM "DBA"."Admin"
WHERE "DBA"."Admin"."last_modified" >= {ml s.last_table_download}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Admin',
	'upload_insert',
'
/* Insert the row into the consolidated database. */
INSERT INTO "DBA"."Admin" ( "admin_id", "data" )
VALUES ( {ml r."admin_id"}, {ml r."data"} )

'
);

CALL ml_add_table_script (
	'https_v1',
	'Admin',
	'upload_delete',
'
/* Delete the row from the consolidated database. */
DELETE FROM "DBA"."Admin"
WHERE "admin_id" = {ml r."admin_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Admin',
	'upload_update',
'
/* Update the row in the consolidated database. */
UPDATE "DBA"."Admin"
SET "data" = {ml r."data"}
WHERE "admin_id" = {ml r."admin_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Admin',
	'upload_fetch_column_conflict',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Admin',
	'upload_old_row_insert',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Admin',
	'download_delete_cursor',
'
SELECT "DBA"."Admin_del"."admin_id"
FROM "DBA"."Admin_del"
WHERE "DBA"."Admin_del"."last_modified" >= {ml s.last_table_download}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Admin',
	'upload_fetch',
'
SELECT "admin_id",
	"data"
FROM "DBA"."Admin"
WITH (UPDLOCK)
WHERE "admin_id" = {ml r."admin_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Admin',
	'upload_new_row_insert',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Admin',
	'resolve_conflict',
'
/* Ignore the uploaded row, so that the consolidated values will win. */

BEGIN END;

'
);

CALL ml_add_table_script (
	'https_v1',
	'Child',
	'download_cursor',
'
SELECT "DBA"."Child"."child_id",
	"DBA"."Child"."parent_id",
	"DBA"."Child"."data"
FROM "DBA"."Child"
WHERE "DBA"."Child"."last_modified" >= {ml s.last_table_download}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Child',
	'upload_insert',
'
/* Insert the row into the consolidated database. */
INSERT INTO "DBA"."Child" ( "child_id", "parent_id", "data" )
VALUES ( {ml r."child_id"}, {ml r."parent_id"}, {ml r."data"} )

'
);

CALL ml_add_table_script (
	'https_v1',
	'Child',
	'upload_delete',
'
/* Delete the row from the consolidated database. */
DELETE FROM "DBA"."Child"
WHERE "child_id" = {ml r."child_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Child',
	'upload_update',
'
/* Update the row in the consolidated database. */
UPDATE "DBA"."Child"
SET "parent_id" = {ml r."parent_id"},
	"data" = {ml r."data"}
WHERE "child_id" = {ml r."child_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Child',
	'upload_fetch_column_conflict',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Child',
	'upload_old_row_insert',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Child',
	'download_delete_cursor',
'
SELECT "DBA"."Child_del"."child_id"
FROM "DBA"."Child_del"
WHERE "DBA"."Child_del"."last_modified" >= {ml s.last_table_download}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Child',
	'upload_fetch',
'
SELECT "child_id",
	"parent_id",
	"data"
FROM "DBA"."Child"
WITH (UPDLOCK)
WHERE "child_id" = {ml r."child_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Child',
	'upload_new_row_insert',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Child',
	'resolve_conflict',
'
/* Ignore the uploaded row, so that the consolidated values will win. */

BEGIN END;

'
);

CALL ml_add_table_script (
	'https_v1',
	'Inventory',
	'download_cursor',
'
SELECT "DBA"."Inventory"."inventory_id",
	"DBA"."Inventory"."data"
FROM "DBA"."Inventory"
WHERE "DBA"."Inventory"."last_modified" >= {ml s.last_table_download}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Inventory',
	'upload_insert',

'
/* Insert the row into the consolidated database. */
INSERT INTO "DBA"."Inventory" ( "inventory_id", "data" )
VALUES ( {ml r."inventory_id"}, {ml r."data"} )

'
);

CALL ml_add_table_script (
	'https_v1',
	'Inventory',
	'upload_delete',
'
/* Delete the row from the consolidated database. */
DELETE FROM "DBA"."Inventory"
WHERE "inventory_id" = {ml r."inventory_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Inventory',
	'upload_update',
'
/* Update the row in the consolidated database. */
UPDATE "DBA"."Inventory"
SET "data" = {ml r."data"}
WHERE "inventory_id" = {ml r."inventory_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Inventory',
	'upload_fetch_column_conflict',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Inventory',
	'upload_old_row_insert',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Inventory',
	'download_delete_cursor',
'
SELECT "DBA"."Inventory_del"."inventory_id"
FROM "DBA"."Inventory_del"
WHERE "DBA"."Inventory_del"."last_modified" >= {ml s.last_table_download}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Inventory',
	'upload_fetch',
'
SELECT "inventory_id",
	"data"
FROM "DBA"."Inventory"
WITH (UPDLOCK)
WHERE "inventory_id" = {ml r."inventory_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Inventory',
	'upload_new_row_insert',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Inventory',
	'resolve_conflict',
'
/* Ignore the uploaded row, so that the consolidated values will win. */

BEGIN END;

'
);

CALL ml_add_table_script (
	'https_v1',
	'Parent',
	'download_cursor',
'
SELECT "DBA"."Parent"."parent_id",
	"DBA"."Parent"."user_id",
	"DBA"."Parent"."data"
FROM "DBA"."Parent"
WHERE "DBA"."Parent"."last_modified" >= {ml s.last_table_download}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Parent',
	'upload_insert',
'
/* Insert the row into the consolidated database. */
INSERT INTO "DBA"."Parent" ( "parent_id", "user_id", "data" )
VALUES ( {ml r."parent_id"}, {ml r."user_id"}, {ml r."data"} )

'
);

CALL ml_add_table_script (
	'https_v1',
	'Parent',
	'upload_delete',
'
/* Delete the row from the consolidated database. */
DELETE FROM "DBA"."Parent"
WHERE "parent_id" = {ml r."parent_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Parent',
	'upload_update',
'
/* Update the row in the consolidated database. */
UPDATE "DBA"."Parent"
SET "user_id" = {ml r."user_id"},
	"data" = {ml r."data"}
WHERE "parent_id" = {ml r."parent_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Parent',
	'upload_fetch_column_conflict',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Parent',
	'upload_old_row_insert',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Parent',
	'download_delete_cursor',
'
SELECT "DBA"."Parent_del"."parent_id"
FROM "DBA"."Parent_del"
WHERE "DBA"."Parent_del"."last_modified" >= {ml s.last_table_download}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Parent',
	'upload_fetch',
'
SELECT "parent_id",
	"user_id",
	"data"
FROM "DBA"."Parent"
WITH (UPDLOCK)
WHERE "parent_id" = {ml r."parent_id"}

'
);

CALL ml_add_table_script (
	'https_v1',
	'Parent',
	'upload_new_row_insert',
	null
);

CALL ml_add_table_script (
	'https_v1',
	'Parent',
	'resolve_conflict',
'
/* Ignore the uploaded row, so that the consolidated values will win. */

BEGIN END;

'
);

CALL ml_add_column (
	'https_v1',
	'Admin',
	null,
	null
);
CALL ml_add_column (
	'https_v1',
	'Admin',
	'admin_id',
	'bigint'
);
CALL ml_add_column (
	'https_v1',
	'Admin',
	'data',
	'varchar'
);
CALL ml_add_column (
	'https_v1',
	'Child',
	null,
	null
);
CALL ml_add_column (
	'https_v1',
	'Child',
	'child_id',
	'bigint'
);
CALL ml_add_column (
	'https_v1',
	'Child',
	'parent_id',
	'bigint'
);
CALL ml_add_column (
	'https_v1',
	'Child',
	'data',
	'varchar'
);
CALL ml_add_column (
	'https_v1',
	'Inventory',
	null,
	null
);
CALL ml_add_column (
	'https_v1',
	'Inventory',
	'inventory_id',
	'bigint'
);
CALL ml_add_column (
	'https_v1',
	'Inventory',
	'data',
	'integer'
);
CALL ml_add_column (
	'https_v1',
	'Parent',
	null,
	null
);
CALL ml_add_column (
	'https_v1',
	'Parent',
	'parent_id',
	'bigint'
);
CALL ml_add_column (
	'https_v1',
	'Parent',
	'user_id',
	'varchar'
);
CALL ml_add_column (
	'https_v1',
	'Parent',
	'data',
	'varchar'
);

COMMIT;

/*------------------------------------------------------------------------------
*                          Procedures to add test data.
*-----------------------------------------------------------------------------*/

create procedure DBA.rep_data(in iter integer default 1,in do_commit integer default 0)
begin
  declare loop_var integer;
  
  set loop_var=0;
  while loop_var < iter loop
    insert into Admin(data, last_modified) values(cast(RAND(*)*20+1 as char(10)), default);
    insert into Parent(user_id,data, last_modified) values('rem100',cast(RAND(*)*20+1 as char(10)), default);
    insert into Child(parent_id,data, last_modified) values(@@identity,cast(RAND(*)*20+1 as char(10)), default);
    set loop_var=loop_var+1;
    if do_commit = 1 then
      commit;
    end if
  end loop;
end;

create procedure DBA.rep_data_inventory(in iter integer default 1,in do_commit integer default 0)
begin
  declare loop_var integer;
  
  set loop_var=0;
  while loop_var < iter loop
    insert into Inventory(data, last_modified) values(loop_var, default);
    set loop_var=loop_var+1;
    if do_commit = 1 then
      commit;
    end if
  end loop;
end;
